@@ -262,10 +262,23 @@ FUNCTION_MAP = {
     277: "DUR2MILLISECONDS",
     278: "STRIPDURATION",
     280: "INTERSECT.RANGES",
+    281: "LATITUDE",
+    282: "LONGITUDE",
+    283: "BEARING",
+    284: "ALTITUDE",
     285: "UNION.RANGES",
     286: "SERIESSUM",
     287: "POLYNOMIAL",
     288: "WEIBULL",
+    289: "CONFIDENCE.T",
+    290: "COVARIANCE.S",
+    291: "MODE.MULT",
+    292: "PERCENTILE.EXC",
+    293: "PERCENTRANK.EXC",
+    294: "QUARTILE.EXC",
+    295: "RANK.AVG",
+    296: "FIND.CASEINSENSITIVE",
+    297: "PLAINTEXT",
     298: "STOCK",
     299: "STOCKH",
     300: "CURRENCY",
@@ -288,6 +301,8 @@ FUNCTION_MAP = {
     319: "TEXTBETWEEN",
     320: "TEXTAFTER",
     321: "REGEX",
+    322: "REFERENCE.NAME",
     323: "FORMULATEXT",
+    324: "REGEX.EXTRACT",
     325: "GETPIVOTDATA",
 }
