@@ -2,7 +2,7 @@ import math
 import re
 
 from array import array
-from datetime import timedelta
+from datetime import timedelta, datetime
 from functools import lru_cache
 from struct import pack, unpack
 from typing import Dict, List
@@ -44,6 +44,14 @@ from numbers_parser.generated import TSPMessages_pb2 as TSPMessages
 from numbers_parser.generated import TSPArchiveMessages_pb2 as TSPArchiveMessages
 from numbers_parser.generated import TSTArchives_pb2 as TSTArchives
 
+SECONDS_IN_HOUR = 60 * 60
+SECONDS_IN_DAY = SECONDS_IN_HOUR * 24
+SECONDS_IN_WEEK = SECONDS_IN_DAY * 7
+
+FORMAT_STYLE_NONE = 0
+FORMAT_STYLE_SHORT = 1
+FORMAT_STYLE_MEDIUM = 2
+
 
 class CellValue:
     def __init__(self, _type):
@@ -55,6 +63,7 @@ class CellValue:
         self.date = EPOCH
         self.d128 = None
         self.bullets = None
+        self.formatted = None
 
     def __repr__(self):  # pragma: no cover
         fields = filter(lambda x: not x.startswith("_"), dir(self))
@@ -144,6 +153,17 @@ class _NumbersModel:
         bds = self.objects[table_id].base_data_store
         return [self.objects[t.tile.identifier] for t in bds.tiles.tiles]
 
+    @lru_cache(maxsize=None)
+    def table_format_entries(self, format_id):
+        return {x.key: x.format for x in self.objects[format_id].entries}
+
+    @lru_cache(maxsize=None)
+    def table_format(self, table_id: int, key: int) -> str:
+        """Return the format assocuated with a format ID for a particular table"""
+        bds = self.objects[table_id].base_data_store
+        format_table_id = bds.format_table.identifier
+        return self.table_format_entries(format_table_id)[key]
+
     @lru_cache(maxsize=None)
     def table_string_entries(self, strings_id):
         return {x.key: x.string for x in self.objects[strings_id].entries}
@@ -152,8 +172,8 @@ class _NumbersModel:
     def table_string(self, table_id: int, key: int) -> str:
         """Return the string assocuated with a string ID for a particular table"""
         bds = self.objects[table_id].base_data_store
-        strings_id = bds.stringTable.identifier
-        return self.table_string_entries(strings_id)[key]
+        table_strings_id = bds.stringTable.identifier
+        return self.table_string_entries(table_strings_id)[key]
 
     def init_table_strings(self, table_id: int):
         """Cache table strings reference and delete all existing keys/values"""
@@ -527,6 +547,30 @@ class _NumbersModel:
         base_data_store.merge_region_map.MergeFrom(
             TSPMessages.Reference(identifier=merge_map_id)
         )
+        # TODO deal with Assertion:
+        # Assert *** Assertion failure #0: -[TSPUnarchiver
+        #   validateReferenceToObjectIdentifier:objectClass:isWeak
+        #   :validateStrongReferences:selector:weakSelector:]
+        #   /Library/Caches/com.apple.xbs/Sources/iWorkDependenciesMacOS/
+        #   iWorkDependenciesMacOS-7034.0.86/shared/persistence/src/TSPUnarchiver.mm:704
+        #   Object [TSTMergeRegionMap-6347027] is not strongly referenced from message
+        #   TST.TableModelArchive-1058328] message type 6001.
+        #   Use TSPReadWeakReferenceMessage<TSTMergeRegionMap>(unarchiver, message,
+        #   completion) instead.
+
+        # component_map = {c.identifier: c for c in self.objects[PACKAGE_ID].components}
+        # component_id = [
+        #     id
+        #     for id, c in component_map.items()
+        #     if c.preferred_locator == "CalculationEngine"
+        # ][0]
+        # component_map[component_id].external_references.append(
+        #     TSPArchiveMessages.ComponentExternalReference(
+        #         component_identifier=table_id,
+        #         object_identifier=merge_map_id,
+        #         is_weak=True,
+        #     )
+        # )
 
     def recalculate_column_row_uid_map(self, table_id: int, data: List):
         table_model = self.objects[table_id]
@@ -553,20 +597,6 @@ class _NumbersModel:
             base_column_row_uids.row_index_for_uid.append(row_num)
             base_column_row_uids.row_uid_for_index.append(row_num)
 
-    def init_table_tile(self, table_id: int, data: List) -> TSTArchives.Tile:
-        base_data_store = self.objects[table_id].base_data_store
-        tile_ids = [t.tile.identifier for t in base_data_store.tiles.tiles]
-        tile = self.objects[tile_ids[0]]
-        tile.maxColumn = 0
-        tile.maxRow = 0
-        tile.numCells = 0
-        tile.numrows = len(data)
-        tile.storage_version = 5
-        tile.last_saved_in_BNC = True
-        tile.should_use_wide_rows = True
-        clear_field_container(tile.rowInfos)
-        return tile
-
     def recalculate_row_info(
         self, table_id: int, data: List, tile_row_offset: int, row_num: int
     ) -> TSTArchives.TileRowInfo:
@@ -575,19 +605,26 @@ class _NumbersModel:
         row_info.tile_row_index = row_num - tile_row_offset
         row_info.cell_count = 0
         cell_storage = b""
+        cell_storage_pre_bnc = b""
 
         if len(data[0]) >= MAX_TILE_SIZE:
             wide_offsets = True
             offsets = [-1] * len(data[0])
+            offsets_pre_bnc = [-1] * len(data[0])
         else:
             wide_offsets = False
             offsets = [-1] * MAX_TILE_SIZE
+            offsets_pre_bnc = [-1] * MAX_TILE_SIZE
         current_offset = 0
+        current_offset_pre_bnc = 0
 
         for col_num in range(len(data[row_num])):
             buffer = self.pack_cell_storage_v5(
                 table_id, data, row_num, col_num, wide_offsets
             )
+            buffer_pre_bnc = self.pack_cell_storage_v3(
+                table_id, data, row_num, col_num, wide_offsets
+            )
             if buffer is not None:
                 cell_storage += buffer
                 if wide_offsets:
@@ -596,20 +633,25 @@ class _NumbersModel:
                     offsets[col_num] = current_offset
                 current_offset += len(buffer)
 
+                cell_storage_pre_bnc += buffer_pre_bnc
+                if wide_offsets:
+                    offsets_pre_bnc[col_num] = current_offset_pre_bnc >> 2
+                else:
+                    offsets_pre_bnc[col_num] = current_offset_pre_bnc
+                current_offset_pre_bnc += len(buffer_pre_bnc)
+
                 row_info.cell_count += 1
 
-        # TODO: only pack as many offsets as last column with data
         row_info.cell_offsets = pack(f"<{len(offsets)}h", *offsets)
+        row_info.cell_offsets_pre_bnc = pack(
+            f"<{len(offsets_pre_bnc)}h", *offsets_pre_bnc
+        )
         row_info.cell_storage_buffer = cell_storage
-        # TODO: do formulas need BNC storage?
-        row_info.cell_offsets_pre_bnc = bytes([0xF0, 0x9F, 0xA4, 0xA0])
-        row_info.cell_storage_buffer_pre_bnc = bytes([0xF0, 0x9F, 0xA4, 0xA0])
+        row_info.cell_storage_buffer_pre_bnc = cell_storage_pre_bnc
         row_info.has_wide_offsets = wide_offsets
         return row_info
 
-    def update_package_metadata(
-        self, obj_id: int, parent: str, locator: str = None, derived=False
-    ):
+    def update_package_metadata(self, obj_id: int, parent: str, locator: str = None):
         component_map = {c.identifier: c for c in self.objects[PACKAGE_ID].components}
         if obj_id not in component_map:
             component_id = [
@@ -634,18 +676,11 @@ class _NumbersModel:
                     )
 
                 self.objects[PACKAGE_ID].components.append(component_info)
-                if derived:
-                    component_map[component_id].external_references.append(
-                        TSPArchiveMessages.ComponentExternalReference(
-                            component_identifier=component_id, object_identifier=obj_id
-                        )
-                    )
-                else:
-                    component_map[component_id].external_references.append(
-                        TSPArchiveMessages.ComponentExternalReference(
-                            component_identifier=obj_id
-                        )
+                component_map[component_id].external_references.append(
+                    TSPArchiveMessages.ComponentExternalReference(
+                        component_identifier=obj_id
                     )
+                )
             else:
                 component_map[component_id].external_references.append(
                     TSPArchiveMessages.ComponentExternalReference(
@@ -746,10 +781,13 @@ class _NumbersModel:
 
     def create_string_table(self):
         table_strings_id, table_strings = self.objects.create_object_from_dict(
-            "Index/Tables/DataList-{}-2",
+            "Index/Tables/DataList-{}",
             {"listType": TSTArchives.TableDataList.ListType.STRING, "nextListID": 1},
             TSTArchives.TableDataList,
         )
+        self.update_package_metadata(
+            table_strings_id, "CalculationEngine", "Tables/DataList-{}"
+        )
         return table_strings_id, table_strings
 
     def style_sheet_map(self, objects):
@@ -780,6 +818,31 @@ class _NumbersModel:
                 "table_name": table_name,
                 "default_row_height": 20.0,
                 "default_column_width": 98.0,
+                "table_style": {"identifier": from_table.table_style.identifier},
+                "body_text_style": {
+                    "identifier": from_table.body_text_style.identifier
+                },
+                "header_row_text_style": {
+                    "identifier": from_table.header_row_text_style.identifier
+                },
+                "header_column_text_style": {
+                    "identifier": from_table.header_column_text_style.identifier
+                },
+                "footer_row_text_style": {
+                    "identifier": from_table.footer_row_text_style.identifier
+                },
+                "body_cell_style": {
+                    "identifier": from_table.body_cell_style.identifier
+                },
+                "header_row_style": {
+                    "identifier": from_table.header_row_style.identifier
+                },
+                "header_column_style": {
+                    "identifier": from_table.header_column_style.identifier
+                },
+                "footer_row_style": {
+                    "identifier": from_table.footer_row_style.identifier
+                },
             },
             TSTArchives.TableModelArchive,
         )
@@ -794,7 +857,42 @@ class _NumbersModel:
         print(f"column_headers_id = {column_headers_id}")
 
         self.update_package_metadata(
-            column_headers_id, "DocumentStylesheet", "Tables/DataList-{}", True
+            column_headers_id, "CalculationEngine", "Tables/HeaderStorageBucket-{}"
+        )
+
+        style_table_id, _ = self.objects.create_object_from_dict(
+            "Index/Tables/DataList-{}",
+            {"listType": TSTArchives.TableDataList.ListType.STYLE, "nextListID": 1},
+            TSTArchives.TableDataList,
+        )
+        print(f"style_table_id = {style_table_id}")
+
+        self.update_package_metadata(
+            style_table_id, "CalculationEngine", "Tables/DataList-{}"
+        )
+
+        formula_table_id, _ = self.objects.create_object_from_dict(
+            "Index/Tables/TableDataList-{}",
+            {"listType": TSTArchives.TableDataList.ListType.FORMULA, "nextListID": 1},
+            TSTArchives.TableDataList,
+        )
+        print(f"formula_table_id = {formula_table_id}")
+
+        self.update_package_metadata(
+            formula_table_id, "CalculationEngine", "Tables/TableDataList-{}"
+        )
+
+        format_table_pre_bnc_id, _ = self.objects.create_object_from_dict(
+            "Index/Tables/TableDataList-{}",
+            {"listType": TSTArchives.TableDataList.ListType.STYLE, "nextListID": 1},
+            TSTArchives.TableDataList,
+        )
+        print(f"format_table_pre_bnc_id = {format_table_pre_bnc_id}")
+
+        self.update_package_metadata(
+            format_table_pre_bnc_id,
+            "DocumentStylesheet",
+            "Tables/TableDataList-{}",
         )
 
         table_model.base_data_store.MergeFrom(
@@ -807,6 +905,11 @@ class _NumbersModel:
                 rowTileTree=TSTArchives.TableRBTree(),
                 columnTileTree=TSTArchives.TableRBTree(),
                 tiles=TSTArchives.TileStorage(tile_size=256, should_use_wide_rows=True),
+                styleTable=TSPMessages.Reference(identifier=style_table_id),
+                formula_table=TSPMessages.Reference(identifier=formula_table_id),
+                format_table_pre_bnc=TSPMessages.Reference(
+                    identifier=format_table_pre_bnc_id
+                ),
             )
         )
 
@@ -826,7 +929,7 @@ class _NumbersModel:
         print(f"row_headers_id = {row_headers_id}")
 
         self.update_package_metadata(
-            row_headers_id, "DocumentStylesheet", "Tables/DataList-{}", True
+            row_headers_id, "CalculationEngine", "Tables/HeaderStorageBucket-{}"
         )
         table_model.base_data_store.rowHeaders.buckets.append(
             TSPMessages.Reference(identifier=row_headers_id)
@@ -834,66 +937,8 @@ class _NumbersModel:
 
         self.recalculate_table_data(table_model_id, data)
 
-        style_table_id, _ = self.objects.create_object_from_dict(
-            "Index/Tables/TableDataList-{}",
-            {"listType": TSTArchives.TableDataList.ListType.STYLE, "nextListID": 1},
-            TSTArchives.TableDataList,
-        )
-        print(f"style_table_id = {style_table_id}")
-
-        self.update_package_metadata(
-            style_table_id, "DocumentStylesheet", "Tables/DataList-{}", True
-        )
-        table_model.base_data_store.styleTable.MergeFrom(
-            TSPMessages.Reference(identifier=style_table_id)
-        )
-
-        formula_table_id, _ = self.objects.create_object_from_dict(
-            "Index/Tables/TableDataList-{}",
-            {"listType": TSTArchives.TableDataList.ListType.FORMULA, "nextListID": 1},
-            TSTArchives.TableDataList,
-        )
-        print(f"formula_table_id = {formula_table_id}")
-
-        self.update_package_metadata(
-            formula_table_id, "DocumentStylesheet", "Tables/DataList-{}", True
-        )
-        table_model.base_data_store.formula_table.MergeFrom(
-            TSPMessages.Reference(identifier=formula_table_id)
-        )
-
-        format_table_pre_bnc_id, _ = self.objects.create_object_from_dict(
-            "Index/Tables/TableDataList-{}",
-            {"listType": TSTArchives.TableDataList.ListType.STYLE, "nextListID": 1},
-            TSTArchives.TableDataList,
-        )
-        print(f"format_table_pre_bnc_id = {format_table_pre_bnc_id}")
-
-        self.update_package_metadata(
-            format_table_pre_bnc_id, "DocumentStylesheet", "Tables/DataList-{}", True
-        )
-        table_model.base_data_store.format_table_pre_bnc.MergeFrom(
-            TSPMessages.Reference(identifier=format_table_pre_bnc_id)
-        )
-
         # TODO: fix package metadata references
 
-        for field in [
-            "table_style",
-            "body_text_style",
-            "header_row_text_style",
-            "header_column_text_style",
-            "footer_row_text_style",
-            "footer_row_text_style",
-            "body_cell_style",
-            "header_row_style",
-            "header_column_style",
-            "footer_row_style",
-        ]:
-            getattr(table_model, field).MergeFrom(
-                TSPMessages.Reference(identifier=getattr(from_table, field).identifier)
-            )
-
         table_info_id, table_info = self.objects.create_object_from_dict(
             "CalculationEngine",
             {},
@@ -1011,7 +1056,65 @@ class _NumbersModel:
 
         return storage[0:length]
 
-    def unpack_cell_storage_v3(self, buffer: bytes) -> CellValue:
+    def pack_cell_storage_v3(
+        self, table_id: int, data: List, row_num: int, col_num: int, wide_offsets: bool
+    ) -> bytearray:
+        """Create a storage buffer for a cell using v3 layout"""
+        cell = data[row_num][col_num]
+        length = 12
+        if isinstance(cell, NumberCell):
+            flags = 0x20
+            length += 8
+            cell_type = TSTArchives.numberCellType
+            value = pack("<d", float(cell.value))
+        elif isinstance(cell, TextCell):
+            flags = 0x10
+            length += 4
+            cell_type = TSTArchives.textCellType
+            value = pack("<i", self.table_string_key(table_id, cell.value))
+        elif isinstance(cell, DateCell):
+            flags = 0x40
+            length += 8
+            cell_type = TSTArchives.dateCellType
+            date_delta = cell.value - EPOCH
+            value = pack("<d", float(date_delta.total_seconds()))
+        elif isinstance(cell, BoolCell):
+            flags = 0x20
+            length += 8
+            cell_type = TSTArchives.boolCellType
+            value = pack("<d", float(cell.value))
+        elif isinstance(cell, DurationCell):
+            flags = 0x20
+            length += 8
+            cell_type = TSTArchives.durationCellType
+            value = value = pack("<d", float(cell.value.total_seconds()))
+        elif isinstance(cell, EmptyCell):
+            return None
+        elif isinstance(cell, MergedCell):
+            return None
+        else:
+            data_type = type(cell).__name__
+            table_name = self.table_name(table_id)
+            warn(
+                f"@{table_name}:[{row_num},{col_num}]: unsupported data type {data_type} for save",
+                UnsupportedWarning,
+            )
+            return None
+
+        storage = bytearray(32)
+        storage[0] = 3
+        storage[2] = cell_type
+        storage[4:8] = pack("<i", flags)
+        storage[12 : 12 + len(value)] = value
+
+        if wide_offsets and len(storage) % 4:
+            padding_len = 4 - (len(storage % 4))
+            length += padding_len
+            storage += bytearray(padding_len)
+
+        return storage[0:length]
+
+    def unpack_cell_storage_v3(self, buffer: bytes, table_id: int) -> CellValue:
         version = buffer[0]
         flags = unpack("<i", buffer[4:8])[0]
         cell_type = buffer[2]
@@ -1038,9 +1141,18 @@ class _NumbersModel:
             seconds = unpack("<d", buffer[offset : offset + 8])[0]
             cell_value.date = EPOCH + timedelta(seconds=seconds)
             offset += 8
+
+        if version > 1 and flags & 0xFF0000 and cell_value.ieee is not None:
+            format_id = unpack("<i", buffer[offset : offset + 4])[0]
+            cell_value.formatted = self.duration_format(
+                cell_value.ieee, format_id, version, flags >> 16, table_id
+            )
+
         return cell_value
 
-    def unpack_cell_storage_v5(self, cell_type: int, buffer: bytes) -> CellValue:
+    def unpack_cell_storage_v5(
+        self, cell_type: int, buffer: bytes, table_id: int
+    ) -> CellValue:
         flags = unpack("<i", buffer[8:12])[0]
         cell_value = CellValue(cell_type)
         offset = 12
@@ -1059,8 +1171,133 @@ class _NumbersModel:
             offset += 4
         if flags & 0x10:
             cell_value.rich = unpack("<i", buffer[offset : offset + 4])[0]
+            offset += 4
+
+        offset += bin(flags & 0x1FE0).count("1") * 4
+
+        if flags & 0x7E000 and cell_value.ieee is not None:
+            format_id = unpack("<i", buffer[offset : offset + 4])[0]
+            cell_value.formatted = self.duration_format(
+                cell_value.ieee, format_id, buffer[0], flags >> 13, table_id
+            )
+        elif flags & 0x7E000 and cell_value.date is not None:
+            format_id = unpack("<i", buffer[offset : offset + 4])[0]
+            cell_value.formatted = self.date_format(
+                cell_value.date, format_id, buffer[0], table_id
+            )
+
         return cell_value
 
+    def duration_format(
+        self,
+        cell_value: int,
+        format_id: int,
+        format_version: int,
+        flags: int,
+        table_id: int,
+    ) -> str:
+        if format_version >= 4:
+            format = self.table_format(table_id, format_id)
+        else:
+            format = self.table_format(table_id, format_id)
+
+        duration_style = format.duration_style
+        if duration_style == -1:
+            return
+        unit_largest = format.duration_unit_largest
+        unit_smallest = format.duration_unit_smallest
+        if format.use_automatic_duration_units:
+            unit_smallest, unit_largest = auto_units(cell_value, format)
+
+        if unit_largest == -1 or unit_smallest == -1:
+            return
+
+        d = cell_value
+        dd = int(cell_value)
+        dstr = []
+
+        if unit_largest == 1:
+            dd = int(d / SECONDS_IN_WEEK)
+            if unit_smallest != 1:
+                d -= SECONDS_IN_WEEK * dd
+            dstr.append(str(dd) + unit_format("week", dd, duration_style))
+        if unit_largest <= 2 and unit_smallest >= 2:
+            dd = int(d / SECONDS_IN_DAY)
+            if unit_smallest > 2:
+                d -= SECONDS_IN_DAY * dd
+            dstr.append(str(dd) + unit_format("day", dd, duration_style))
+        if unit_largest <= 4 and unit_smallest >= 4:
+            dd = int(d / SECONDS_IN_HOUR)
+            if unit_smallest > 4:
+                d -= SECONDS_IN_HOUR * dd
+            dstr.append(str(dd) + unit_format("hour", dd, duration_style))
+        if unit_largest <= 8 and unit_smallest >= 8:
+            dd = int(d / 60)
+            if unit_smallest > 8:
+                d -= 60 * dd
+            if duration_style == FORMAT_STYLE_NONE:
+                pad = (unit_largest == 8 and unit_smallest == 8) or dd > 10
+                dstr.append(("" if pad else "0") + str(dd))
+            else:
+                dstr.append(str(dd) + unit_format("minute", dd, duration_style))
+        if unit_largest <= 16 and unit_smallest >= 16:
+            dd = int(d)
+            if unit_smallest > 16:
+                d -= dd
+            if duration_style == FORMAT_STYLE_NONE:
+                pad = (unit_smallest == 16 and unit_largest == 16) or dd >= 10
+                dstr.append(("" if pad else "0") + str(dd))
+            else:
+                dstr.append(str(dd) + unit_format("second", dd, duration_style))
+        if unit_smallest >= 32:
+            dd = int(round(1000 * d))
+            if duration_style == FORMAT_STYLE_NONE:
+                padding = "0" if dd >= 10 else "00"
+                padding = "" if dd >= 100 else padding
+                dstr.append(f"{padding}{dd}")
+            else:
+                dstr.append(
+                    str(dd) + unit_format("millisecond", dd, duration_style, "ms")
+                )
+        duration_str = (":" if duration_style == 0 else " ").join(dstr)
+        if duration_style == FORMAT_STYLE_NONE:
+            duration_str = re.sub(r":(\d\d\d)$", r".\1", duration_str)
+        return duration_str
+
+    def date_format(
+        self,
+        cell_value: datetime,
+        format_id: int,
+        format_version: int,
+        table_id: int,
+    ) -> str:
+        if format_version >= 4:
+            format = self.table_format(table_id, format_id)
+        else:
+            format = self.table_format(table_id, format_id)
+
+        format = format.date_time_format
+        format = format.replace("a", "%p")
+        format = format.replace("EEEE", "%A")
+        format = format.replace("EEE", "%a")
+        format = re.sub(r"\by\b", "%Y", format)
+        format = format.replace("yyyy", "%Y")
+        format = format.replace("yy", "%y")
+        format = format.replace("MMMM", "%B")
+        format = format.replace("MMM", "%b")
+        format = format.replace("MM", "%m")
+        format = format.replace("M", "%-m")
+        format = re.sub(r"\bd\b", "%-d", format)
+        format = format.replace("dd", "%d")
+        format = format.replace("HH", "%H")
+        format = format.replace("h", "%-I")
+        format = format.replace("mm", "%M")
+        format = format.replace("ss", "%S")
+        if format == "":
+            return ""
+
+        return cell_value.strftime(format)
+
     @lru_cache(maxsize=None)
     def table_formulas(self, table_id: int):
         return TableFormulas(self, table_id)
@@ -1075,9 +1312,9 @@ class _NumbersModel:
         cell_type = buffer[1]
         cell_value = CellValue(cell_type)
         if buffer[0] <= 3:
-            cell_value = self.unpack_cell_storage_v3(buffer)
+            cell_value = self.unpack_cell_storage_v3(buffer, table_id)
         else:
-            cell_value = self.unpack_cell_storage_v5(cell_type, buffer)
+            cell_value = self.unpack_cell_storage_v5(cell_type, buffer, table_id)
 
         if cell_value.type == TSTArchives.numberCellType or cell_value.type == 10:
             cell_value.value = cell_value.d128
@@ -1348,3 +1585,52 @@ def unpack_decimal128(buffer: bytearray) -> float:
         mantissa = -mantissa
     value = mantissa * 10**exp
     return float(value)
+
+
+def unit_format(unit: str, value: int, style: int, abbrev: str = None):
+    plural = "" if value == 1 else "s"
+    if abbrev is None:
+        abbrev = unit[0]
+    if style >= FORMAT_STYLE_MEDIUM:
+        return f" {unit}" + plural
+    elif style == FORMAT_STYLE_SHORT:
+        return f"{abbrev}"
+    else:
+        return ""
+
+
+def auto_units(cell_value, format):
+    unit_largest = format.duration_unit_largest
+    unit_smallest = format.duration_unit_smallest
+
+    if cell_value == 0:
+        unit_largest = 2
+        unit_smallest = 2
+    else:
+        if cell_value >= SECONDS_IN_WEEK:
+            unit_largest = 1
+        elif cell_value >= SECONDS_IN_DAY:
+            unit_largest = 2
+        elif cell_value >= SECONDS_IN_HOUR:
+            unit_largest = 4
+        elif cell_value >= 60:
+            unit_largest = 8
+        elif cell_value >= 1:
+            unit_largest = 16
+        else:
+            unit_largest = 32
+
+        if math.floor(cell_value) != cell_value:
+            unit_smallest = 32
+        elif cell_value % 60:
+            unit_smallest = 16
+        elif cell_value % SECONDS_IN_HOUR:
+            unit_smallest = 8
+        elif cell_value % SECONDS_IN_DAY:
+            unit_smallest = 4
+        elif cell_value % SECONDS_IN_WEEK:
+            unit_smallest = 2
+        if unit_smallest < unit_largest:
+            unit_smallest = unit_largest
+
+    return unit_smallest, unit_largest
